<?php

/**
 * Import Table: PredefinedCategory
 *
 * @author Nicolás Ibarra Sabogal
 */
class SWIFT_ImportTable_Quickreply_cat extends SWIFT_ImportTable
{
	/**
	 * Constructor
	 *
	 * @author Nicolás Ibarra Sabogal
	 * @param SWIFT_ImportManager $_SWIFT_ImportManagerObject The Import Manager Object
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __construct(SWIFT_ImportManager $_SWIFT_ImportManagerObject)
	{
		parent::__construct($_SWIFT_ImportManagerObject, 'Quickreply_cat');

		if (!$this->TableExists('quickreply_cat'))
		{
			$this->SetByPass(true);

			return false;
		}

		SWIFT_Loader::LoadModel('Macro:MacroCategory', APP_TICKETS);
		SWIFT_Loader::LoadModel('Macro:MacroReply', APP_TICKETS);
		SWIFT_Loader::LoadLibrary('Macro:MacroManager', APP_TICKETS);

		return true;
	}

	/**
	 * Destructor
	 *
	 * @author Nicolás Ibarra Sabogal
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __destruct()
	{
		parent::__destruct();

		return true;
	}

	/**
	 * Import the data based on offset in the table
	 *
	 * @author Nicolás Ibarra Sabogal
	 * @return int The number of records on success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function Import()
	{
		if (!$this->GetIsClassLoaded())
		{
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);

			return false;
		}

		$_staffCache = $this->Cache->Get('staffcache');

//		if ($this->GetOffset() == 0)
//		{
//			$this->Database->Query("DELETE FROM " . TABLE_PREFIX . "macrocategories");
//		}

		$_count = 0;

		$this->DatabaseImport->QueryLimit("SELECT * FROM quickreply_cat ORDER BY id ASC", $this->GetItemsPerPass(), $this->GetOffset());
		while ($this->DatabaseImport->NextRecord())
		{
			$_count++;

			$_newStaffID = $this->ImportManager->GetImportRegistry()->GetKey('staff', $this->DatabaseImport->Record['techid']);

			$_categoryType = SWIFT_MacroCategory::TYPE_PUBLIC;
			if ($this->DatabaseImport->Record['global'] == '0')
			{
				$_categoryType = SWIFT_MacroCategory::TYPE_PRIVATE;
			}

			$this->GetImportManager()->AddToLog('Importing Macro Category: ' . htmlspecialchars($this->DatabaseImport->Record['name']), SWIFT_ImportManager::LOG_SUCCESS);

			$this->Database->AutoExecute(TABLE_PREFIX . 'macrocategories',
					array('parentcategoryid' => '0', 'categorytype' => intval($_categoryType), 'staffid' => intval($_newStaffID),
						'restrictstaffgroupid' => '0', 'title' => $this->DatabaseImport->Record['name']), 'INSERT');
			$_macroCategoryID = $this->Database->InsertID();

			$this->ImportManager->GetImportRegistry()->UpdateKey('macrocategory', $this->DatabaseImport->Record['id'], $_macroCategoryID);
		}

		return $_count;
	}

	/**
	 * Retrieve the total number of records in a table
	 *
	 * @author Nicolás Ibarra Sabogal
	 * @return int The Record Count
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	protected function GetTotal()
	{
		if (!$this->GetIsClassLoaded())
		{
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);

			return false;
		}

		$_countContainer = $this->DatabaseImport->QueryFetch("SELECT COUNT(*) AS totalitems FROM quickreply_cat");
		if (isset($_countContainer['totalitems']))
		{
			return $_countContainer['totalitems'];
		}

		return 0;
	}

	/**
	 * Retrieve the number of items to process in a pass
	 *
	 * @author Nicolás Ibarra Sabogal
	 * @return int The Number of Items
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function GetItemsPerPass()
	{
		if (!$this->GetIsClassLoaded())
		{
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);

			return false;
		}

		return 500;
	}
}
?>